/*
	example program which demonstrates how to 
	query Net for adaptor information
*/




#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/kernel.h>
#include <sys/name.h>

#include "net41msg.h"


#define	NET_INTERFACES	10	/* number of logical lans	*/


main()
{

		/* NET_DRVR_INFO request/reply msg buffer
			used to query Net for its interfaces */
		/* _filler is used to make sure there is enough room 
			to hold the di[num_drvrs] of the reply, which is
			defined as di[] in net41msg.h */
	union
	{
		struct _net_drvr_info		s;
		struct _net_drvr_info_reply	r;
		char _filler[sizeof(struct _net_drvr_info_reply) +
				NET_INTERFACES * sizeof(struct _net_drvr_info_struct)];
	}net_info;


		/* NET_RNODEMAP request/reply msg buffer
			used to query Net for it's netmap */
		/* _filler is used to make sure there is enough room
			to hold the card[num_card] of the reply, which is
			defined as card[] in net41msg.h */
	union
	{
		struct _net_rnodemap		s;
		struct _net_rnodemap_reply	r;
		char _filler[sizeof(struct _net_rnodemap_reply) +
				NET_INTERFACES * sizeof(struct _card_info)];
	}net_map_info;

	struct _net_drvr_info_struct	*net_drvr_info;
	struct _card_info				*net_card_info;

	pid_t	net_pid;
	int		i, j, num_interfaces;





	net_pid= qnx_name_locate(0, "qnx/net", 0, NULL);
	if(net_pid == -1)
	{
		fprintf(stderr, "Unable to locate Net manager\n");
		exit(-1);
	}


		/* query Net for its node mappings */
	memset(&net_map_info, 0, sizeof(net_map_info));
	net_map_info.s.type= _NET_RNODEMAP;
	net_map_info.s.log_nid= getnid();
	if(Send(net_pid, &net_map_info.s, &net_map_info.r,
		sizeof(net_map_info.s), sizeof(net_map_info)) == -1)
	{
		fprintf(stderr, "Send RNODEMAP to Net(%d) failed\n", net_pid);
		exit(-1);
	}
	if(net_map_info.r.status != EOK)
	{
		errno= net_info.r.status;
		perror("Send(NET_RNODEMAP)");
		exit(-1);
	}


		/* query Net for info on its drivers/interfaces	*/
	memset(&net_info, 0, sizeof(net_info));
	net_info.s.type= _NET_DRVR_INFO;
	if(Send(net_pid, &net_info.s, &net_info.r,
		sizeof(net_info.s), sizeof(net_info)) == -1)
	{
		fprintf(stderr, "Send DRVR_INFO to Net(%d) failed\n", net_pid);
		exit(-1);
	}
	if(net_info.r.status != EOK)
	{
		errno= net_info.r.status;
		perror("Send(NET_DRVR_INFO)");
		exit(-1);
	}

	/* start processing returned information */

	num_interfaces= net_info.r.num_drvrs;
	printf("Net configured for %d lans...searching for adaptors\n\n", num_interfaces);
	if(num_interfaces > NET_INTERFACES)		/* make sure we stay within the msg buffer */
		num_interfaces= NET_INTERFACES;

		/* loop thru checking for inuse interface slots */
	net_drvr_info= net_info.r.di;
	for(i=0; i<num_interfaces; i++, net_drvr_info++)
		if(net_drvr_info->lan_addr != 0)	/* lan in use */
		{
			printf("slot %d used for lan %d\n", i, net_drvr_info->lan_addr);
			printf("adaptor type: [%d]  sub-type: [%d]  (see net_drvr.h)\n",
				net_drvr_info->hw_type, net_drvr_info->hw_subtype);
			printf("driver is pid %d\n", net_drvr_info->pid);

				/* loop thru node mappings to find this cards address */
			net_card_info= net_map_info.r.card;
			for(j=0; j<num_interfaces; j++, net_card_info++)
			{
				if(net_card_info->lan_addr == net_drvr_info->lan_addr)
				{
					printf("adaptor physical address: %X.%X.%X.%X.%X.%X\n\n",
						net_card_info->phys_addr[0], net_card_info->phys_addr[1],
						net_card_info->phys_addr[2], net_card_info->phys_addr[3],
						net_card_info->phys_addr[4], net_card_info->phys_addr[5]);
					break;
				}
			}
		}
}
