/*	Raw Net Interface example.

	register with Net on all lans
	if -s option (start), send a broadcast
	forever
		rx packet
		tx packet

	This example needs 2 qnx nodes to demonstrate.
	On node 1, run:
		# netraw

	Next, on node 2, run:
		# netraw -s

	...and watch them go!

*/
#include "rx_pkt.h"
#include "tx_pkt.h"
#include "net41msg.h"
#include "net_nq.h"
#include <sys/kernel.h>
#include <sys/name.h>
#include <sys/seginfo.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>



/* how many packets to buffer */
#define	TXPKTS	100
#define	RXPKTS	100	

/* default protocol */
unsigned short	proto= 0xbeef;

/* the important infomation */
pid_t			net_pid;
short unsigned	net_ds;
short unsigned	net_handle;
void far 		*net_free_q;
void far		*net_tx_q;
short unsigned 	net_q_seg;
pid_t			net_proxy;

/* counters */
int				rx_cnt;
int				tx_cnt;

/* buffer to tx */
#define	BUF_LEN	1500
char buffer[BUF_LEN]= {"XXblah blah blah blah blah blah"};

/* helper functions */

void
print_progress(){
	printf("rx_cnt: %d\ttx_cnt: %d\trx_drop: %d\ttx_fail: %d\n",
		rx_cnt, tx_cnt, rx_dropped, tx_failed);
}

void
print_pkt_info(rxpkt_t *rxpkt){
	char *p;
	printf("RX len=%d lan=%d proto=%2.2X%2.2X\n", 
			rxpkt->len, rxpkt->lan, rxpkt->data[0], rxpkt->data[1]);
	p= rxpkt->from_addr;
	printf("src addr: %2.2X.%2.2X.%2.2X.%2.2X.%2.2X.%2.2X\n",
		p[0], p[1], p[2], p[3], p[4], p[5]);
	p= rxpkt->to_addr;
	printf("dst addr: %2.2X.%2.2X.%2.2X.%2.2X.%2.2X.%2.2X\n",
		p[0], p[1], p[2], p[3], p[4], p[5]);
	printf("pkt data: [%s]\n\n", &rxpkt->data[2]);
}

/* deregistering is very important ! */
void
dereg(){
	struct _net_raw_dreg			dereg;

	dereg.handle= net_handle;
	dereg.type= _NET_RAW_DREG;

	Send(net_pid, &dereg, NULL, sizeof(dereg), 0);	
	print_progress();
}

void
got_signal( int signo ){
	dereg();
	_exit(signo);
}


/* transmit a packet */
void
tx( char *data, int len, mac_t addr, int lan ){
	struct _mxfer_entry32_16 far *mx_ptr;
	txpkt_t	*txpkt;

	txpkt= get_txpkt();
	txpkt->qpkt= q_get_first(net_free_q, 1, net_q_seg);
	_fmemset(txpkt->qpkt, 0, sizeof(struct _net_nq_pkt));
	txpkt->qpkt->type= _VC_RAW;
	txpkt->qpkt->dst_vid= lan;
	txpkt->qpkt->src_vid= net_handle;
	txpkt->qpkt->mx_seg= net_ds;
	txpkt->qpkt->mx_off= FP_OFF(txpkt->mx);
	txpkt->qpkt->mx_parts= 1;
	_fmemcpy(txpkt->qpkt->phys_dst_nid, addr, 6);
	mx_ptr= MK_FP(net_ds, txpkt->mx);
	_setmx32_16(txpkt->mx, MK_FP(net_ds, buffer), BUF_LEN);
	if(q_put_last(net_tx_q, txpkt->qpkt, 1, net_q_seg) == 1)
		Trigger(net_proxy);
}






void
main(int argc, char **argv){
	int	start= 0;
	int	verbose= 1;
	int	opt;
	pid_t pid;
	struct _net_raw_reg	reg;
	struct _net_raw_reg_reply reply;
	rxpkt_t	*rxpkt;
	txpkt_t *txpkt;
	struct _net_nq_pkt far *qpkt;
	mac_t	broadcast={ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
	int	broadcast_lan=1;
	int i;

	/* first need to do some initialization */


	while((opt= getopt(argc, argv, "hsvqp:")) != -1)
		switch(opt){
			case 's':	start++; break;
			case 'v':	verbose++; break;
			case 'q':	verbose--; break;
			case 'p':	proto= atoi(optarg); break;
			case 'h':
				printf("%s [-vqsh] [-p ]\n", argv[0]);
				printf("\t-h\thelp (this message)\n");
				printf("\t-v\tincrease verbosity (max= -vvv)\n");
				printf("\t-q\tdecrease verbosity\n");
				printf("\t-p proto (default 0xbeef)\n");
				printf("\t-s\tstart -- send a broadcast to kick things off\n");
				exit(1);
		}

	/* stuff the protocol into buffer for tx'ing */
	buffer[0]= proto >> 8;
	buffer[1]= proto;


	net_pid= qnx_name_locate(0, "qnx/net", 0, NULL);
	if(net_pid == -1){
		fprintf(stderr, "Unable to locate Net manager\n");
		exit(-1);
	}

	if(rx_init(RXPKTS) == -1){
		perror("rx_init");
		exit(-1);
		}

	if(tx_init(TXPKTS) == -1){
		perror("tx_init");
		exit(-1);
		}

	/* do we need another page of data before arming ? */
	free(malloc(4096));
	qnx_segment_arm(net_pid, -1, 0);

	memset(&reg, 0, sizeof(reg));	
	memset(&reply, 0, sizeof(reply));	

	/* prepare to register... */
	reg.type= _NET_RAW_REG;
	reg.rpt_len= 2;
	reg.rpt[0]= (proto & 0xff00) >> 8;
	reg.rpt[1]= (proto & 0x00ff);
	reg.rx_lan_addr= -1;
	reg.rx_alloc_fn_ofs= FP_OFF(rx_getbuf);
	reg.rx_done_fn_ofs= FP_OFF(rx_done);
	reg.tx_done_fn_ofs= FP_OFF(tx_done);
	reg.rx_alloc_fn_seg= 
	reg.rx_done_fn_seg=
	reg.tx_done_fn_seg= FP_SEG(tx_done);
	reg.txd_q_ofs= FP_OFF(&txd_q);
	reg.ds= my_ds();
	reg.txd_q_seg= FP_SEG(&txd_q);

	if(Send(net_pid, &reg, &reply, sizeof(reg), sizeof(reply)) == -1){
		perror("Send(NET_RAW_REG)");
		exit(-1);
		}

	if(reply.status != EOK){
		errno= reply.status;
		perror("Reply(NET_RAW_REG)");
		exit(-1);
		}

	/* now registered, make sure we deregister before terminating */
	atexit(dereg);
	
	for(i=_SIGMIN; i<_SIGMAX; i++)
		signal(i, got_signal);

	/* save all the important stuff... */
	net_handle= reply.handle;
	net_ds= reply.ds;
	net_q_seg= reply.net_qseg;
	net_free_q= MK_FP(net_q_seg, reply.free_q);
	net_tx_q= MK_FP(net_q_seg, reply.work_q);
	net_proxy= reply.net_mid;


	/* kick things off ? */
	if(start){
		if(verbose)
			printf("start mode: sending broadcast message on lan %d\n", broadcast_lan);
			/* here we are a bit lazy... just blindly try lan 1 */
			/* if we get a tx error with ESRCH, increment and try again */
		tx(buffer, strlen(buffer), broadcast, broadcast_lan);
		}



	/* spin rx/tx'ing packets until one gets dropped... */
	for(;;){
		pid= Receive(0,0,0);
		if(pid == -1)
			exit(-1);
		else if(pid == rx_done_proxy){
			rxpkt= get_rxpkt();
			if(rxpkt==NULL){
				printf("RX error\n");
				continue;
				}
			if(verbose>3)
				print_pkt_info(rxpkt);
			tx(buffer, BUF_LEN, rxpkt->from_addr, rxpkt->lan);
			put_rxpkt(rxpkt);
			rx_cnt++;
		}else if(pid == tx_done_proxy){
				tx_cnt++;
				qpkt= q_get_first(&txd_q, 1, net_q_seg);
				if(qpkt==NULL){
					printf("TX error\n");
					tx_failed++;
					continue;
					}
				if(qpkt->type == EOK){
					if(verbose>3){
						printf("TX'd ok...\n");
						}
				}else if(qpkt->type == EIO){
					tx_failed++;
					printf("TX'd error...\n");
				}else{
					tx_failed++;
					errno= qpkt->type;
					perror("errno");
						/* for lazy kick */
					if(start && rx_cnt==0 && qpkt->type==ESRCH){
						printf("start mode: sending broadcast message on lan %d\n", ++broadcast_lan);
						tx(buffer, strlen(buffer), broadcast, broadcast_lan);
						}
				}
				txpkt= (void *)qpkt->mx_off;
				put_txpkt(txpkt);
		}
		if(verbose)
			print_progress();
	}
}
