#include "rx_pkt.h"


#ifndef NULL
#define NULL 0
#endif

/* these MUST be protected via disable/enable because they
	are shared between Net and the application				*/
static rxpkt_t *rxhead, *rxtail;	/* list of rx'd pkts -- head & tail	*/
static rxpkt_t *rxfree;				/* list of free pkts				*/

static rxpkt_t	*rxcur;				/* pkt being rx'd -- not on any list
										and only used by Net callbacks	*/


int	rx_dropped;						/* number of dropped pkts on recv	*/
pid_t rx_done_proxy;

extern	short	net_ds;


/* called during initialization before regestering with Net	*/
int
rx_init( int n )
{
	int i;
	rxpkt_t *r;
	
	rx_done_proxy= qnx_proxy_attach(0,0,0,24);
	if(rx_done_proxy == -1){
		perror("rx_done_proxy");
		exit(-1);
		}
	for(i=0; i<n; i++){
		r= (rxpkt_t *)malloc(sizeof(rxpkt_t));
		if(!r)
			return -1;
		memset(r, 0, sizeof(rxpkt_t));
		r->next= rxfree;
		rxfree= r;
		}
	return 0;
}
		



/* application functions to get/free rx'd pkts		*/


/* called by application to grab oldest rx packet	*/
rxpkt_t *
get_rxpkt()
{
	rxpkt_t *r= NULL;

	disable();			/* have to protect */
		if(rxhead){
			r= rxhead;
			rxhead= rxhead->next;
			if(rxhead==NULL)	
				rxtail= NULL;
		}
	enable();
	return r;
}

/* called by application to re-use rx packet */
void
put_rxpkt( rxpkt_t *r )
{
	disable();		/* have to protect */
		r->next= rxfree;
		rxfree= r;
	enable();
}
			



/* rx callbacks executed by Net manager						*/


/* a far function which returns a far pointer.				*/
/* passed during registration as .rx_alloc_fn_ofs field in 
	_net_raw_reg message									*/
/* returns an address to copy the pkt into--NULL to drop it	*/
/* parameters m1,m2,m3 represent the 6 byte physical
    address of the destination						*/
void far * far
rx_getbuf(int length, short m1, short m2, short m3)
{
	if(rxcur){
		/* already rx'ing -- should not happen	*/
		/* log error							*/
		rx_dropped++;
		return NULL;
	}

	if(length > RX_LEN){
		/* data too large -- log error			*/
		rx_dropped++;
		return NULL;
	}


	disable();		/* have to protect -- keep it short	*/
		rxcur= rxfree;
		if(rxcur)
			rxfree= rxfree->next;
	enable();

	if(rxcur){
		short *p;
		void far *fp;

		rxcur->len= length;				/* store length of packet	*/
		p= (short *)rxcur->to_addr;		/* store physical address	*/
		*p++ = m1;						/*   of destination 		*/
		*p++ = m2;
		*p++ = m3;
			/* has to be addressable by Net */
		fp=(MK_FP(net_ds, &(rxcur->data)));
		return fp;
	}else{
		/* no buffer availiable -- log error	*/
		rx_dropped++;
		return NULL;
	}
}


/* passed during registration as .rx_done_fn_ofs field in 
	_net_raw_reg message									*/
/* rx'd packet now in rcur->data							*/
/* store lan and mac info in rcur, put rcur into end of rx list	*/
/* parameters m1,m2,m3 represent the 6 byte physical
    address of the sender, parameter lanid is the lan
    the packet was recieved on                              */
/* return rx_done_proxy -- Net will trigger this			*/
int far
rx_done(short lanid, short m1, short m2, short m3)
{
	short *p;

	rxcur->lan= lanid;					/* store lan				*/
	p= (short *)rxcur->from_addr;		/* store physical address	*/
	*p++ = m1;							/*	of sender				*/
	*p++ = m2;
	*p++ = m3;
	rxcur->next= NULL;

	disable();		/* put at end of rx list */
		if(rxtail)
			rxtail->next= rxcur;
		else		/* if no tail, must be empty (no head either) */
			rxhead= rxcur;
		rxtail= rxcur;
		rxcur= NULL;
	enable();
		
		/* wake up application */
	return rx_done_proxy;
}
